using System;
using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

using Citi.SpreadEngine.UserControls;

namespace Citi.SpreadEngine
{
	/// <summary>
	/// Summary description for OverrideSettingsControlCell.
	/// </summary>
	public class OverrideSettingsControlCell : GridGenericControlCellModel
	{
		public OverrideSettingsControlCell(GridModel parentGridModel) : base(parentGridModel){}

		public override GridCellRendererBase CreateRenderer(GridControlBase grid)
		{
			return (new OverrideSettingsControlCellRenderer(grid,this));
		}
		
		public class OverrideSettingsControlCellRenderer : GridGenericControlCellRenderer
		{
			private OverrideSettingsControl activeControl;
			private OverrideSettingsControl staticControl;

			public OverrideSettingsControlCellRenderer(GridControlBase grid, GridCellModelBase model):base(grid,model)
			{
				// Get the column index that this renderer is in, and work out the mapping name
				// to pass to OverrideSettingsControl
				GridDataBoundGrid gdbg = grid as GridDataBoundGrid;
				activeControl= new OverrideSettingsControl(gdbg);
				staticControl= new OverrideSettingsControl(gdbg);
//				grid.DefaultRowHeight=activeControl.Height;
				grid.DefaultColWidth=activeControl.Width;
				MakeUnBuffered(staticControl);
			}

			private void MakeUnBuffered(Control c)
			{
				System.Reflection.MethodInfo mInfo = c.GetType().GetMethod("SetStyle", System.Reflection.BindingFlags.Instance |
					System.Reflection.BindingFlags.InvokeMethod |
					System.Reflection.BindingFlags.NonPublic);

				if (!(mInfo == null))
				{
					mInfo.Invoke(c, new object[]{ControlStyles.DoubleBuffer, false});
				}

				if (c.Controls.Count>0)
				{
					foreach (Control c1 in c.Controls)
					{
						c1.CausesValidation = false;
						MakeUnBuffered(c1); // recursive call
					}
				}
			}

			protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
			{
				clientRectangle.Inflate(-1,-1);

				if (ShouldDrawFocused(rowIndex,colIndex))
					style.Control=activeControl;
				else 
					style.Control=staticControl;

				base.OnDraw(g,clientRectangle,rowIndex,colIndex,style);
			}
		}

	}
}
